<?php

/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 **/

namespace MAM\Daemon\Engine\Frostbite3\Rcon;
/**
 * Rcon Class for Frostbite Games
 * With this class you can send rcon commands to a bf server
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Rcon {

    /**
     * Server IP address
     * @var String
     */
    protected $address;

    /**
     * Server Query Port
     * @var int
     */
    protected $port;

    /**
     * Resource of the socket
     * @var resource
     */
    protected $socket;

    /**
     * Rcon password of the server
     * @var string
     */
    protected $rcon_password;

    /**
     * Sequence number of the packet
     * @var int
     */
    protected $sequence = 1;

    /**
     * If class receives the servers logging
     * @var bool
     */
    protected $rcon_log;

    /**
     * Constructs the Rcon Class and calls connect to connect to the server
     * @param string $address
     * @param int $port
     */
    public function __construct($address, $port) {
        $this->address = $address;
        $this->port = $port;
        $this->connect();
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'Frostbite Rcon Class';
    }

    /**
     * Connects to the server, if failed throws exception
     * @throws \Exception
     */
    private function connect() {
        if (is_resource($this->socket)) {
            $this->quit();
        }
        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        socket_set_option($this->socket, SOL_SOCKET, SO_RCVTIMEO, array('sec' => 120, 'usec' => 0));
        if (socket_connect($this->socket, $this->address, $this->port) === false) {
            throw new \Exception('Failed to establish a socket connection with server ' . $this->address . ':' . $this->port, 00);
        }
    }

    /**
     * Closes the socket
     */
    private function quit() {
        if (is_resource($this->socket)) {
            socket_close($this->socket);
        }
    }

    /**
     * Sends a encoded message to the server, if error with sending throws exception
     * @param string $str
     * @throws \Exception
     */
    private function send($str) {
        if (socket_write($this->socket, $str) === false) {
            throw new \Exception('Sending to socket failed', 00);
        }
    }

    /**
     * Receives sth from the server and returns it, if connection is lost throws exception
     * @param int $length
     * @return string
     * @throws \Exception
     */
    private function receive($length) {
        $return = socket_read($this->socket, $length);
        if (socket_last_error($this->socket) == 110) {
            throw new \Exception('Trying to restart because of a connection timeout');
        }
        return $return;
    }

    /**
     * Encodes the Integer to an Int32
     * @param int $int
     * @return string
     */
    private function encodeInt32($int) {
        return pack('I', $int);
    }

    /**
     * Encodes the sequence number as the header
     * @param $sequence
     * @return string
     */
    private function encodeHeader($sequence) {
        $header = $sequence & 0x3fffffff;
        return pack('I', $header);
    }

    /**
     * Encodes all words of the array
     * @param $words
     * @return array
     */
    private function encodeWords($words) {
        $size = 0;
        $encoded_words = '';
        foreach ($words as $word) {
            $encoded_words .= $this->encodeInt32(mb_strlen($word));
            $encoded_words .= $word;
            $encoded_words .= "\x00";
            $size += mb_strlen($word) + 5;
        }
        return array($size, $encoded_words);
    }

    /**
     * Encodes the whole packet
     * @param int $sequence
     * @param array $words
     * @return string
     */
    private function encodePacket($sequence, $words) {
        $encoded_header = $this->encodeHeader($sequence);
        $encoded_count_words = $this->encodeInt32(count($words));
        list($words_size, $encoded_words) = $this->encodeWords($words);
        $encoded_size = $this->encodeInt32($words_size + 12);
        return $encoded_header . $encoded_size . $encoded_count_words . $encoded_words;
    }

    /**
     * Encodes the client request
     * @param array|string $rconCommand
     * @return string
     */
    private function encodeRequest($rconCommand) {
        if (is_array($rconCommand)) {
            $words = $rconCommand;
        } else {
            $string = $rconCommand;
            if ((strpos($string, '"') !== false) || (strpos($string, '\'') !== false)) {
                $words = preg_split('/["\']/', $string);
                $words_count = count($words);
                for ($i = 0; $i < $words_count; $i++) {
                    $words[$i] = trim($words[$i]);
                }
            } else {
                $words = preg_split('/\s+/', $string);
            }
        }
        foreach ($words as $key => $value) {
            if ($value === null || $value == '') {
                unset($words[$key]);
            }
        }
        $packet = $this->encodePacket($this->sequence, $words);
        $this->sequence = $this->sequence + 1;
        return $packet;
    }

    /**
     * decodes an Int32
     * @param  string $data
     * @return bool
     */
    private function decodeInt32($data) {
        if ($data == '') {
            return false;
        }
        $decode = unpack('I', mb_substr($data, 0, 4));
        return $decode[1];
    }

    /**
     * Decodes the header
     * @param $data
     * @return array
     */
    private function decodeHeader($data) {
        $value = $this->decodeInt32($data);
        $sequence = $value & 0x3FFFFFFF;
        $isFromServer = ($value & 0x80000000) != 0;
        $isResponse = ($value & 0x40000000) != 0;
        return array($isFromServer, $isResponse, $sequence);
    }

    /**
     * decodes the words from the body of the packet
     * @param $size
     * @param $data
     * @return array
     */
    private function decodeWords($size, $data) {
        $words = array();
        $offset = 0;
        while ($offset < $size) {
            $word_len = $this->decodeInt32(mb_substr($data, $offset, 4));
            $word = mb_substr($data, $offset + 4, $word_len);
            array_push($words, trim($word));
            $offset += $word_len + 5;
        }
        return $words;
    }

    /**
     * Returns a full packet
     * @return array
     */
    private function receivePacket() {
        $buffer = $this->receive(8);
        $length = $this->decodeInt32(mb_substr($buffer, 4, 4));
        if($length > 2000) {
            $now = 0;
            while($length > ($now + 12)) {
                $buffer .= $this->receive(2000);
                $now = mb_strlen(mb_substr($buffer, 12));
            }
        } else {
            $buffer .= $this->receive($this->decodeInt32(mb_substr($buffer, 4, 4)));
        }
        $body = $this->decodeWords($this->decodeInt32(mb_substr($buffer, 4, 4)) - 12, mb_substr($buffer, 12));
        return $body;
    }

    /**
     * Sends an rcon command to the server, if it fails it throws an exception
     * @param $command
     * @return array
     * @throws \Exception
     */
    public function rconCommand($command) {
        $encoded_command = $this->encodeRequest($command);
        $this->send($encoded_command);
        $body = $this->receivePacket();
        if (!$this->getStatus($body)) {
            if (is_array($command)) {
                $command = implode(' ', $command);
            }
            throw new \Exception('rcon command failed: ' . $command . ': ' . $body[0], 01);
        }
        return $body;
    }

    /**
     * Receives log packages
     * @return array
     */
    public function receiveLogPackets() {
        $buffer = $this->receive(8);
        $buffer .= $this->receive($this->decodeInt32(mb_substr($buffer, 4, 4)) - 8);
        $body = $this->decodeWords($this->decodeInt32(mb_substr($buffer, 4, 4)) - 12, mb_substr($buffer, 12));
        return $body;
    }

    /**
     * Returns the status of an Packet
     * @param String $data
     * @return boolean
     */
    public function getStatus($data) {
        return (isset($data[0]) && $data[0] == 'OK') ? true : false;
    }

    /**
     * Logs onto the server
     * @throws \Exception
     */
    public function login() {
        if ($this->rcon_password === false) {
            throw new \Exception('Please specify a rcon password with setRconPassword() before you log in');
        }
        $decoded_salt_request = $this->encodeRequest('login.hashed');
        $this->send($decoded_salt_request);
        $salt = $this->str2hex($this->receivePacket()[1]);
        $hashedPassword = md5($salt . $this->rcon_password, true);
        $decoded_login_request = $this->encodeRequest('login.hashed ' . strtoupper($this->hex2str($hashedPassword)));
        $this->send($decoded_login_request);
        $return = $this->receivePacket();
        if (!$this->getStatus($return)) {
            throw new \Exception('Login failed: ' . $return[1]);
        }
    }

    /**
     * Converts a string in hex.
     * @param String $str
     * @return string
     */
    private function str2hex($str) {
        $str = str_replace(array(' ', '\x'), '', $str);
        $hex = pack('H*', $str);
        return $hex;
    }

    /**
     * Converts a hex into a string.
     * @param String $hex
     * @return string
     */
    private function hex2str($hex) {
        $str = unpack('H*', $hex);
        return array_shift($str);
    }

    /**
     * Sets the rcon password for this connection
     * @param string $pw
     */
    public function setRconPassword($pw) {
        $this->rcon_password = $pw;
    }

    /**
     * Sets if the class is set to log or not
     * @param $set
     */
    public function setLog($set) {
        $this->rcon_log = $set;
        if ($set === true) {
            socket_set_nonblock($this->socket);
        } else {
            socket_set_block($this->socket);
        }
    }

    /**
     * Returns the status if the sever sends the logs or not
     * @return bool
     */
    public function getLog() {
        return $this->rcon_log;
    }

    /**
     * Returns the host address
     * @return string
     */
    public function getAddress() {
        return $this->address;
    }

    /**
     * Returns the host port
     * @return int
     */
    public function getPort() {
        return $this->port;
    }
}