<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3;

use MAM\Daemon\Core\Registry;

/**
 * ActionHandler Class for Frostbite
 *
 * This class contains all ActionHandler relevant functions
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class ActionHandler {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Core\Mod
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     */
    protected $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;

    /**
     * Constructor of the class.
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = &$registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
    }

    /**
     * Log action: player joins the server.
     * @param array $parts
     */
    public function Join($parts) {
        //0:GUID
        //1:PID
        //2:Name

        if (array_key_exists($parts[0], $this->players)) {
            //Player already on server
            $this->players[$parts[0]]->update($parts[1], $parts[2]);
        } else {
            $this->players[$parts[0]] = new Player($parts[0], $parts[1], $parts[2]);
            $search = array(
                '<NAME>',
                '<PID>',
                '<GUID>',
                '<PBGUID>',
                '<IP>',
                '<GROUP>'
            );
            $replace = array(
                $this->players[$parts[0]]->getName(),
                $this->players[$parts[0]]->getPid(),
                $parts[0],
                substr($parts[0], -8),
                $this->players[$parts[0]]->getIP(),
                $this->players[$parts[0]]->getGroup(),
            );
            $string = str_replace($search, $replace, "<GROUP> <NAME> (<PID>) has joined. Guid:<GUID>, IP:<IP>");
            if (trim($string) != "") {
                $this->logging->info($string);
            }
            //Can't trigger this event in constructor, because then $players[guid] isn't set already, we have to wait for this
            $this->mod->triggerEvent("playerJoined", $parts[0]);
        }
    }

    /**
     * Log action: player joins team.
     * @param array $parts
     */
    public function JoinTeam($parts) {
        //JT => jointeam => JT;GUID;PID;TEAM;NICKNAME;
        //0:GUID
        //1:PID
        //2:TEAM
        //3:Name
        //4:null
        list($guid, $pid, $team, $name, $dummy) = $parts;

        if (!array_key_exists($parts[0], $this->players)) {
            $this->logging->warning("JOINTEAM by player '$name' who is actually not on the server: PID: $pid, GUID: $guid");
            return;
        }

        $this->players[$guid]->update($pid, $name);
        $oldteam = $this->players[$guid]->getTeam();
        $this->players[$guid]->setTeam($team);
        //Is playerChangeTeam now
        //$this->mod->triggerEvent("playerJoinTeam", $guid, $oldteam);
    }

    /**
     * Log action: player leaves the server.
     * @param array $parts
     */
    public function Quit($parts) {
        //0:GUID
        //1:PID
        //2:Name
        if (!array_key_exists($parts[0], $this->players)) {
            $this->logging->warning("QUIT by player '$parts[2]' who is actually not on the server: PID: $parts[1], GUID: $parts[0]");
            return;
        }
        if ($parts[1] != $this->players[$parts[0]]->getPID()) {
            $this->logging->warning("IGNORING QUIT by player '$parts[2]' because PIDs are not the same: PID: $parts[1], GUID: $parts[0]");
            return;
        }
        //Can't trigger this event in destructor, because then $players[guid] is unset already, it has to be triggered before
        $this->mod->triggerEvent("playerQuit", $parts[0]);
        $this->logging->info("Player '$parts[2]' quit, PID: $parts[1], GUID: $parts[0]");
        $this->players[$parts[0]]->cleanup();
        unset($this->players[$parts[0]]);
    }

    /**
     * Log action: player says something, if message starts with command prefix, the command will be executed.
     * @param array $parts
     */
    public function Say($parts) {
        //0:GUID
        //1:PID
        //2:Name
        //3:Msg

        if (!array_key_exists($parts[0], $this->players)) {
            $this->logging->warning("SAY by player '$parts[3]' who is actually not on the server: PID: $parts[1], GUID: $parts[0]");
            return;
        }

        $this->players[$parts[0]]->update($parts[1], $parts[2]);

        $executed = false;
        if ($parts[3][0] == $this->mod->getCV("daemon", "cmdprefix")) {
            $this->mod->executeCommand($parts[3], $parts[0], $executed);
        }

        $this->mod->triggerEvent("playerSay", array($parts[0], $parts[3], $executed));
    }

    /**
     * Log action: next map / map restart
     *
     * @param string $dvars
     */
    public function NextMap($dvars) {
        $this->logging->info("Next map / map restart");

        //Reconnect of rcon connection (try max. 3 times on error)
        for ($try = 1; $try <= 3; $try ++) {
            try {
                $this->rcon->rconConnect();
                break;
            } catch (\Exception $e) {
                $this->logging->warning("Reconnect try $try: {$e->getMessage()}");
                sleep(1);

                if ($try >= 3) {
                    $this->logging->error("Something is wrong with the rcon-connection (see above)");
                }
            }
        }
        $this->logging->info("RCON connection established (" . $this->mod->getCV("connection", "ip") . ":" . $this->mod->getCV("connection", "port") . ")");

        if (!array_key_exists("_manuadminmod", $dvars)) {
            $this->serverRestart();
        }

        $this->gametype = $dvars["g_gametype"];
        $this->mod->lastmaps[] = $dvars["mapname"];

        $this->logging->info("Current map: " . $dvars["mapname"] . " (" . $dvars["g_gametype"] . ")");

        foreach ($this->players as $key => $value) {
            $this->players[$key]->resetTeam();
        }

        sleep(2); //Just wait until map has loaded

        if ($this->mod->getCV("quake3", "fixguidrelax")) {
            $this->rcon->rcon("pb_sv_guidrelax 0");
        }


        $this->mod->updateTeamNames();

        //load map specific config files
        if (is_file($this->mod->getConfigDir() . "maps/default.json")) {
            $this->mod->readConfigFile("maps/default.json");
        }
        if (is_file($this->mod->getConfigDir() . "maps/" . $dvars["mapname"] . ".json")) {
            $this->mod->readConfigFile("maps/" . $dvars["mapname"] . ".json");
        }

        if (is_readable($this->mod->getConfigDir() . "maps/default.rcon")) {
            $this->logging->info("Executing: maps/default.rcon");
            $cmds = file($this->mod->getConfigDir() . "maps/default.rcon");
            foreach ($cmds as $cmd) {
                $cmd = preg_replace('|(//).*|i', '', $cmd);
                if (!empty($cmd)) {
                    $this->rcon->rconRcon(trim($cmd));
                }
            }
        }

        if (is_readable($this->mod->getConfigDir() . "maps/" . $dvars["mapname"] . ".rcon")) {
            $this->logging->info("Executing: maps/" . $dvars["mapname"] . ".rcon");
            $cmds = file($this->mod->getConfigDir() . "maps/" . $dvars["mapname"] . ".rcon");
            foreach ($cmds as $cmd) {
                $cmd = preg_replace('|(//).*|i', '', $cmd);
                if (!empty($cmd)) {
                    $this->rcon->rconRcon(trim($cmd));
                }
            }
        }

        $this->mod->triggerEvent("nextMap", $dvars);
    }

    /**
     * Runs when a server restart ist detected.
     */
    public function serverRestart() {
        if (!Registry::getInstance()->iceops) {
            $this->logging->info("Detected a server restart");
            $this->players = array();
            $this->mod->setVersionDvar();

            $this->mod->triggerEvent("serverRestart");
        } else {
            $this->mod->setVersionDvar();
        }
    }

    /**
     * Log action: player kills another player (or himself ;-)
     * @param array $parts
     */
    public function Kill($parts) {
        //K;c90dc162f630049cec0cb632e5e0aa50;0;;manu;87216e65d8efd209b1addfb667c2a3dc;1;;FN! ZZ;m21_mp;147;MOD_HEAD_SHOT;head
        //0 1                                2 34    5                                6 78      9      10  11            12
        //KILL;GUID KILLED;PID KILLED; ?!; KILLED NAME; KILLER GUID; KILLER PID; ?!;KILLERNAME;WEAPON;DMG; IMAGE; BODY PART
        //0    1           2           3   4            5            6            7  8         9      10   11     12
        // Ich glaube 3 und 7 sollten die Teams sein, leider ham die Programmierer da was verpeilt
        // Erster Teil (K) wird vom Parser entfernt
        //   0             1            2             3             4             5            6             7             8        9         10      11
        list($victim_guid, $victim_pid, $victim_team, $victim_name, $killer_guid, $killer_pid, $killer_team, $killer_name, $weapon, $damage, $image, $bodypart) = $parts;

        if ($victim_guid != "" && !array_key_exists($victim_guid, $this->players)) {
            $this->logging->warning("DEATH by player '$victim_name' who is actually not on the server: PID: $victim_pid, GUID: $victim_guid");
            return;
        }

        if ($killer_guid != "" && !array_key_exists($killer_guid, $this->players)) {
            $this->logging->warning("DEATH by player '$killer_name' who is actually not on the server: PID: $killer_pid, GUID: $killer_guid");
            return;
        }

        //Team is not logged in cod4
        $noteamupdate = array("cod4", /* "cod5" */);

        if ($victim_pid != -1) {
            $this->players[$victim_guid]->update($victim_pid, $victim_name);
            if (!in_array($this->mod->getGame(), $noteamupdate)) {
                $this->players[$victim_guid]->setTeam($victim_team);
            }
        }
        if ($killer_pid != -1) {
            $this->players[$killer_guid]->update($killer_pid, $killer_name);
            if (!in_array($this->mod->getGame(), $noteamupdate)) {
                $this->players[$killer_guid]->setTeam($killer_team);
            }
        }

        //Decide if (self/team) kill
        if ($killer_pid == -1) {
            $type = "selfkill";
        } elseif ($this->players[$killer_guid]->getTeam() != "none" && $this->players[$victim_guid]->getTeam() != "none" && $this->players[$killer_guid]->getTeam() == $this->players[$victim_guid]->getTeam() && $this->mod->getGame() != "dm") {
            $type = "teamkill";
        } else {
            $type = "kill";
        }

        $this->players[$victim_guid]->death($type, $killer_guid, array($weapon, $image), $damage, $bodypart);
        if ($killer_pid != -1) {
            $this->players[$killer_guid]->kill($type, $victim_guid, array($weapon, $image), $damage, $bodypart);
        }

        //$this->mod->triggerEvent("kill", array($type, $victim_guid, $killer_guid, array($weapon, $image), $damage, $bodypart));
    }

    /**
     * Log action: player damages another player (or himself ;-))
     * @param array $parts
     */
    public function Damage($parts) {
        list($victim_guid, $victim_pid, $victim_team, $victim_name, $attacker_guid, $attacker_pid, $attacker_team, $attacker_name, $weapon, $damage, $image, $bodypart) = $parts;

        if ($victim_guid != "" && !array_key_exists($victim_guid, $this->players)) {
            $this->logging->warning("DAMAGETAKEN by player '$victim_name' who is actually not on the server: PID: $victim_pid, GUID: $victim_guid");
            return;
        }

        if ($attacker_guid != "" && !array_key_exists($attacker_guid, $this->players)) {
            $this->logging->warning("DAMAGEGIVEN by player '$attacker_name' who is actually not on the server: PID: $attacker_pid, GUID: $attacker_guid");
            return;
        }


        if ($victim_pid != -1) {
            $this->players[$victim_guid]->update($victim_pid, $victim_name);
            $this->players[$victim_guid]->setTeam($victim_team);
        }
        if ($attacker_guid != $victim_guid && $attacker_guid != "") {
            $this->players[$attacker_guid]->update($attacker_pid, $attacker_name);
            $this->players[$attacker_guid]->setTeam($attacker_team);
        }

        //Decide if (self/team) kill
        if ($attacker_guid == $victim_guid || $attacker_guid == "") {
            $type = "selfdamage";
        } elseif ($this->players[$attacker_guid]->getTeam() == $this->players[$victim_guid]->getTeam() && $this->mod->getGameType() != "dm") {
            $type = "teamdamage";
        } else {
            $type = "damage";
        }

        $this->players[$victim_guid]->damageTaken($type, $attacker_guid, array($weapon, $image), $damage, $bodypart);
        if ($attacker_pid != -1) {
            $this->players[$attacker_guid]->damageGiven($type, $victim_guid, array($weapon, $image), $damage, $bodypart);
        }

        //$this->mod->triggerEvent("damage", array($type, $victim_guid, $attacker_guid, array($weapon, $image), $damage, $bodypart));
    }

    /**
     * Log action: vehicle damage
     * @param array $parts
     */
    public function VehicleDamage($parts) {
        list($entityid, $vehicle_team, $attacker_guid, $attacker_pid, $attacker_team, $attacker_name, $weapon, $damage, $image, $bodypart) = $parts;

        //VD;903;;;-1;world;;defaultweapon_mp;21497;MOD_EXPLOSIVE;none
        if ($attacker_guid == "") {
            return;
        }

        if (!array_key_exists($attacker_guid, $this->players)) {
            $this->logging->warning("VEHICLEDAMAGEGIVEN by player '$attacker_name' who is actually not on the server: PID: $attacker_pid, GUID: $attacker_guid");
            return;
        }

        $this->players[$attacker_guid]->update($attacker_pid, $attacker_name);
        $this->players[$attacker_guid]->setTeam($attacker_team);

        if ($attacker_team == $vehicle_team && $this->mod->getGameType() != "dm") {
            $type = "teamdamage";
        } else {
            $type = "damage";
        }

        $this->players[$attacker_guid]->vehicleDamageGiven($type, $entityid, array($weapon, $image), $damage, $bodypart);
    }

    /**
     * Log action: actors damage ???
     * @param array $parts
     */
    public function ActorsDamage($parts) {
        list($actor_id, $actor_team, $attacker_guid, $attacker_pid, $attacker_team, $attacker_name, $weapon, $damage, $image, $bodypart) = $parts;

        if ($attacker_guid != "" && !array_key_exists($attacker_guid, $this->players)) {
            $this->logging->warning("ACTORSDAMAGEGIVEN by player '$attacker_name' who is actually not on the server: PID: $attacker_pid, GUID: $attacker_guid");
            return;
        }

        if ($attacker_guid != "") {
            $this->players[$attacker_guid]->update($attacker_pid, $attacker_name);
            $this->players[$attacker_guid]->setTeam($attacker_team);
        }

        if ($actor_team == $attacker_team && $this->mod->getGameType() != "dm") {
            $type = "teamdamage";
        } else {
            $type = "damage";
        }

        $this->players[$attacker_guid]->actorsDamageGiven($type, $entityid, array($weapon, $image), $damage, $bodypart);
    }

    /**
     * Log action: Win/loss/tie
     * @param string $result win|loss|result
     * @param array $parts
     */
    public function Result($result, $parts) {

        //Differences between cod5 and cod2, cod4 doesn't log this
        //1236256169 W;1209692573;1;M4DM4N
        list($guid, $pid, $name) = $parts;

        if (!array_key_exists($guid, $this->players)) {
            $this->logging->warning("RESULT (w,l,t) by player '$name' who is actually not on the server: PID: $guid, GUID: $pid");
            return;
        }

        $this->players[$guid]->update($pid, $name);
        //Events: playerWin, playerLoss, playerTie
        $this->mod->triggerEvent("player" . ucfirst($result), $guid);
    }

    /**
     * Log action: Various actions (flagcaptured, hqcaptures, bombplanted, ...)
     * @param string $action
     * @param array $parts
     */
    public function Action($action, $parts) {
        //FC => flagCaptured => FC;GUID;PID;NICKNAME
        list($guid, $pid, $name) = $parts;

        $this->players[$guid]->update($pid, $name);

        $events = array(
            "flagcaptured" => "playerFlagCaptured",
            "flagtaken" => "playerFlagTaken",
            "flagreturned" => "playerFlagReturned",
            "hqcaptures" => "playerHqCaptured",
            "hqdestroyed" => "playerHqDestroyed",
            "bombplanted" => "playerBombPlanted",
            "bombdefused" => "playerBombDefused"
        );

        $this->mod->triggerEvent($events[$action], $guid);
    }
}
