<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Quake3;

/**
 * Quake Log File Parser Class
 *
 * This class parses the logfiles of quake games
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Parser {
    /**
     * Logfile
     *
     */
    private $logfile = false;
    /**
     * Parsed log lines
     *
     */
    private $parsedloglines = 0;
    /**
     * Parsed bytes
     *
     */
    private $parsedbytes = 0;
    /**
     * Last parsed
     *
     */
    private $lastparsed = 0;
    /**
     * Timestamp type
     *
     */
    private $timestamptype = "relative";
    /**
     * Logpos
     *
     */
    protected $logpos = false;

    /**
     * Constructs the object
     * @param String $logfile The file to games_mp.log
     * @param boolean $success
     * @return boolean
     */
    public function __construct($logfile, &$success) {
        if (!is_readable($logfile)) {
            $success = false;
            return;
        }
        $this->logfile = $logfile;
        $this->logpos = filesize($this->logfile);

        $success = true;
    }

    /**
     * Get the new lines to parse
     * @return array The new lines
     */
    public function getNewLines() {
        //clearstatcache();
        if (filesize($this->logfile) < $this->logpos) {
            //Return pointer if log file was deleted and recreated
            $this->logpos = 0;
            \MAM\Daemon\Core\Registry::getInstance()->logging->warning("Filepointer was bigger than filesize");
        }

        $fp = fopen($this->logfile, "r");
        fseek($fp, $this->logpos, SEEK_SET);

        $newlines = array();


        while (!feof($fp)) {
            $get = fgets($fp);
            if ($get === false) {
                break;
            }
            $this->logpos += strlen($get);
            $newlines[] = $get;
        }

        fclose($fp);

        return $newlines;
    }

    /**
     * Parse the new lines
     * @param String $line The LogLine
     * @return boolean false
     */
    public function parseLine($line) {
        $pattern_abs = '|^\s*(\d+) |';
        $pattern_rel = '|^\s*(\d+):(\d{2}) |';

        if (preg_match($pattern_rel, $line, $subpatterns)) {
            $this->timestamptype = "relative";
            $timestamp = 60 * $subpatterns[1] + $subpatterns[2];
        } elseif (preg_match($pattern_abs, $line, $subpatterns)) {
            $this->timestamptype = "absolute";
            $timestamp = $subpatterns[1];
        } else {
            return false;
        }


        $lasttime = $this->lastparsed;
        $thistime = $subpatterns[1] = $timestamp;

        $this->lastparsed = $subpatterns[1];
        $this->parsedloglines ++;
        $this->parsedbytes += strlen($line);

        $line = trim(str_replace($subpatterns[0], "", $line));

        $action = $this->getAction($line);

        return array(
            "timestamp" => $this->lastparsed,
            "line" => $line,
            "action" => $action,
            "parsed" => $this->parseParts($line, $action),
        );
    }

    /**
     * Find out which action it is
     * @param String $line the logline
     * @return string The Action
     */
    private function getAction($line) {
        $actions = array(
            "damage" => "D;",
            "kill" => "K;",
            "join" => "J;",
            "quit" => "Q;",
            "say" => "say;",
            "sayteam" => "sayteam;",
            "weapon" => "Weapon;",
            "nextmap" => "InitGame:",
            "exitmap" => "ShutdownGame:",
            "mapend" => "ExitLevel: executed",
            "vehicledamage" => "VD;",
            "actordamage" => "AD;",
            "jointeam" => "JT;",
            "tie" => "T;",
            "win" => "W;",
            "loss" => "L;",
            "flagcaptured" => "FC;",
            "flagtaken" => "FT;",
            "flagreturned" => "FR;",
            "hqcaptures" => "RC;",
            "hqdestroyed" => "RD;",
            "bombplanted" => "BP;",
            "bombdefused" => "BD;",
            "action" => "A;"
        );
        foreach ($actions as $key => $value) {
            if (strpos($line, $value) === 0) {
                return $key;
            }
        }
        return "unknown";
    }

    /**
     * Find out what to do
     *
     * @param string $line
     * @param string $action
     * @return boolean
     */
    private function parseParts($line, $action) {
        switch ($action) {
            case "damage":
            case "weapon":
            case "kill":
            case "join":
            case "quit":
            case "vehicledamage":
            case "actordamage":
            case "jointeam":
            case "tie":
            case "win":
            case "loss":
            case "flagcaptured":
            case "flagtaken":
            case "flagreturned":
            case "hqcaptures":
            case "hqdestroyed":
            case "bombplanted":
            case "bombdefused":
            case "action":
                $parts = explode(";", $line);
                array_shift($parts);
                return $parts;

            case "say":
            case "sayteam";
                $parts = explode(";", $line, 5);
                array_shift($parts);
                if ($parts[3][0] == chr(21)) {
                    $parts[3] = substr($parts[3], 1);
                }
                return $parts;

            case "mapend":
            case "exitmap":
                return false;

            case "nextmap":
                $explode = explode("\\", $line);
                $parts = array();
                $explode_list = count($explode);
                for ($i = 1; $i < $explode_list; $i += 2) {
                    $parts[$explode[$i]] = $explode[$i + 1];
                }
                return $parts;

            default:
                return false;
        }
    }

    /**
     * Returns the last parsed time stamp
     *
     * @return mixed
     */
    public function getLastParsedTimeStamp() {
        return $this->lastparsed;
    }

    /**
     * Returns the timestamp type
     * @return String
     */
    public function getTimeStampType() {
        return $this->timestamptype;
    }

    /**
     * Returns the Parsed Log Lines
     * @return int
     */
    public function getParsedLogLines() {
        return $this->parsedloglines;
    }

    /**
     * Returns the parsed bytes
     * @return int
     */
    public function getParsedBytes() {
        return $this->parsedbytes;
    }
}
