<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Quake3\Rcon;

/**
 * Rcon Class for Quake Games
 *
 * With this class you can send rcon commands to a cod server
 *
 * @author Manu
 */
class Rcon {
    /**
     * Server IP Adress
     * @var String
     */
    protected $address;
    /**
     * Server Query Port
     * @var int
     */
    protected $port;
    /**
     * Rcon Password
     * @var String
     */
    private $rconpassword = false;
    /**
     * Connection Pointer
     * @var fp
     */
    private $fp;
    /**
     * Last Ping
     * @var String
     */
    private $lastPing = false;

    /**
     * Timeout value for stream_set_timeout in milliseconds
     *
     * Lower Value makes Rcon connection faster.
     * Default cod games are limited to 7e5 which is about 0.75 seconds
     *
     * @var int Timeout Value. "e" means exponent
     */
    private $rcon_timeout = 7e5;

    /**
     * Constructor
     *
     * @param string $address ip address
     * @param int $port
     */
    public function __construct($address, $port) {
        $this->address = $address;
        $this->port = $port;

        $this->connect();
    }

    /**
     * Sets the rcon password.
     * @param string $pw
     */
    public function setRconpassword($pw) {
        $this->rconpassword = $pw;
    }

    /**
     * Executes a rcon command and returns the response.
     * @param string $str command
     * @return boolean|string response or false if rcon password is not set.
     */
    public function rcon($str) {
        if (!$this->rconpassword) {
            return false;
        }
        $this->send("rcon " . $this->rconpassword . " $str");
        return $this->getResponse();
    }

    /**
     * Sends the request into the socket.
     * @param string $str
     */
    private function send($str) {
        fwrite($this->fp, "\xFF\xFF\xFF\xFF$str\x00");
    }

    /**
     * Returns the response from last command.
     * @return string
     */
    private function getResponse() {
        stream_set_timeout($this->fp, 0, $this->rcon_timeout);
        $s = '';
        $start = microtime(true);
        do {
            $read = fread($this->fp, 9999);
            $s .= substr($read, strpos($read, "\n") + 1);
            if (!isset($end)) {
                $end = microtime(true);
            }
            $info = stream_get_meta_data($this->fp);
        } while (!$info["timed_out"]);

        $this->lastPing = round(($end - $start) * 1000);

        return $s;
    }

    /**
     * Closes the rcon connection.
     *
     * @return boolean
     */
    public function quit() {
        if (is_resource($this->fp)) {
            fclose($this->fp);
            return true;
        }
        return false;
    }

    /**
     * Connects to the gameserver.
     * @throws Exception
     */
    public function connect() {
        if (is_resource($this->fp)) {
            $this->quit();
        }

        $this->fp = fsockopen("udp://$this->address", $this->port, $errno, $errstr, 5);
        if (!$this->fp) {
            throw new \Exception("RCON connection failed: $errstr ($errno)");
        }
    }

    /**
     * Returns the game status (getstatus).
     * @return boolean|array false if response is empty, otherwise an array with dvars and players
     */
    public function getGameStatus() {
        $this->send("getstatus");
        $response = $this->getResponse();

        if (empty($response)) {
            return false;
        }

        list($dvarslist, $playerlist) = explode("\n", $response, 2);

        $dvarslist = explode("\\", $dvarslist);
        $dvars = array();
        $dvarlistcount = count($dvarslist);
        for ($i = 1; $i < $dvarlistcount; $i += 2) {
            $dvars[$dvarslist[$i]] = $dvarslist[$i + 1];
        }

        $playerlist = explode("\n", $playerlist);
        array_pop($playerlist);
        $players = array();
        foreach ($playerlist as $value) {
            list($score, $ping, $name) = explode(" ", $value, 3);
            $players[] = array(
                "name" => substr($name, 1, -1),
                "score" => $score,
                "ping" => $ping
            );
        }

        return array($dvars, $players);
    }

    /**
     * Returns the game info (getinfo).
     * @return boolean|array false if response is empty, otherwise array with dvars
     */
    public function getGameInfo() {
        $this->send("getinfo");
        $response = $this->getResponse();

        if (empty($response)) {
            return false;
        }

        $dvarslist = explode("\\", $response);
        $dvars = array();
        $dvarlist_count = count($dvarslist);
        for ($i = 1; $i < $dvarlist_count; $i += 2) {
            $dvars[$dvarslist[$i]] = $dvarslist[$i + 1];
        }

        return $dvars;
    }

    /**
     * Returns last ping.
     * @return int
     */
    public function getLastPing() {
        return $this->lastPing;
    }

    /**
     * Changes the wait time for a response
     *
     * Thos is usefull because normal cod servers can only handle 1 request each 0.7 seconds.
     * With a patched version it's possible to send over 80 commands in 0.7 seconds
     *
     * @param bool $use_fastrcon If Serverfiles are patched
     */
    public function changeRconSpeed($use_fastrcon) {
        if ($use_fastrcon || \MAM\Daemon\Core\Registry::getInstance()->iceops) {
            $this->rcon_timeout = 1e4;
        } else {
            $this->rcon_timeout = 7e5;
        }
    }
}
