<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\Logging;

use MAM\Daemon;

/**
 * LogHandler handles the Logfiles
 *
 * Each object creates a new logfile
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class LogHandler implements Daemon\Libraries\Logging\LogInterface {
    /**
     * Filepointer to the file
     * @var type
     */
    private $file_pointer;
    /**
     * Path to the logfile
     * @var string
     */
    private $log_path;
    /**
     * Echo log to the console?
     * @var boolean
     */
    private $echo = true;

    /**
     * Opens the files and check if the file has write permissions
     * @param String $filepath Path to the logfile. Without ending .log  . For example 'plugins/chatlog/'
     * @param String $filename Name of the logfile without file ending. For example  chat
     */
    public function __construct($filepath, $filename) {
        $this->openFile($filepath, $filename);
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Loghandler Class ({$this->log_path})";
    }

    /**
     * Enable or Disable the Echo Mode for Logfiles
     * @param boolean $bool
     */
    public function setEcho($bool) {
        if (is_bool($bool)) {
            $this->echo = $bool;
        } else {
            $this->echo = true;
            $this->warning("Echomode must be boolean (true / false");
        }
    }

    /**
     * Sets the log rotation type
     * @param string $type
     */
    public function setLogrotate($type) {
        if ($type == 'day') {
            \MAM\Daemon\Core\Registry::getInstance()->jobs->addCronJob(1, 'm', -1, array($this, 'checkLogRotate'));
        }
    }

    /**
     * Checks if time is 0:00 and rotate the log
     */
    public function checkLogRotate() {
        if (date('H:i') == '00:00') {
            fclose($this->fp);
            rename($this->log_path, substr($this->log_path, -4) . '-' . date('Y-m-d') . 'log');
            $this->reOpenFile();
        }
    }

    /**
     * Critical Errors
     *
     * Example: Use of Config Vars which aren't defined and haven't a default Var
     *
     * @param string $message
     * @param array $context
     * @return null
     */
    public function error($message, array $context = array()) {
        $line = date("[d.m.y H:i:s]") . " " . ucfirst(LogLevel::ERROR) . ': ' . $message;
        $this->write($line);

        if (DEBUG) {
            $e = new \Exception;
            foreach (Daemon\Libraries\ErrorHandler\ErrorHandler::getPHPTrace($e) as $line) {
                $this->debug($line);
            }
        }
        exit(1);
    }

    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message
     * @param array $context
     * @return null
     */
    public function warning($message, array $context = array()) {
        $line = date("[d.m.y H:i:s]") . " " . ucfirst(LogLevel::WARNING) . ': ' . $message;
        $this->write($line);
    }

    /**
     * Normal but significant events.
     *
     * @param string $message
     * @param array $context
     * @return null
     */
    public function notice($message, array $context = array()) {
        $line = date("[d.m.y H:i:s]") . " " . ucfirst(LogLevel::NOTICE) . ': ' . $message;
        $this->write($line);
    }

    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message
     * @param array $context
     * @return null
     */
    public function info($message, array $context = array()) {
        $line = date("[d.m.y H:i:s]") . " " . ucfirst(LogLevel::INFO) . ': ' . $message;
        $this->write($line);
    }

    /**
     * Detailed debug information.
     *
     * @param string $message
     * @param array $context
     * @return null
     */
    public function debug($message, array $context = array()) {
        if (DEBUG) {
            $line = date("[d.m.y H:i:s]") . " " . ucfirst(LogLevel::DEBUG) . ': ' . $message;
            $this->write($line);
        }
    }

    /**
     * Writes one or multiple lines to the logfile
     *
     * @param String/Array $string  one or multiple chat lines. Lines as array are also possible
     * @param boolean $echo Echo the logline to the screen
     * @return boolean True if successful
     */
    public function write($string, $echo = false) {
        $this->reOpenFile();

        fwrite($this->file_pointer, $string . PHP_EOL);


        if ($echo || $this->echo) {
            echo (strncasecmp(PHP_OS, 'WIN', 3) == 0) ? $string . PHP_EOL : self::colorOutput($string) . PHP_EOL;
        }

        return true;
    }

    /**
     * Close the filepointer
     */
    public function __destruct() {
        if (is_resource($this->file_pointer)) {
            fclose($this->file_pointer);
        }
    }

    /**
     * Reopen the file if fp is gone
     */
    private function reOpenFile() {
        if (!$this->file_pointer || !is_resource($this->file_pointer)) {
            $this->file_pointer = fopen($this->log_path, "a");
        }
    }

    /**
     * opens the files and check if the file is writeable
     * @param String $filepath The path to the file
     * @param String $filename The name of the file
     */
    private function openFile($filepath, $filename) {

        $logdir = Daemon\Core\Registry::getInstance()->logdir;

        $filepath = strtolower($filepath);
        $filename = strtolower($filename);


        /** Check if dir is writeable* */
        if (!is_writable($logdir)) {
            exit('Error: The folder ' . $logdir . " is not writeable. Please change the permissions of this folder" . PHP_EOL);
        }

        if (!is_dir($logdir . $filepath)) {
            if (!mkdir($logdir . $filepath, 0770, True)) {
                exit('Error: Can\'t create folder ' . $logdir . $filepath . ". Maybe Folder is not writeable, or you don't own the files" . PHP_EOL);
            }
        }

        $filename = $filename . ".log";

        $this->log_path = $logdir . $filepath . $filename;


        $this->file_pointer = fopen($logdir . $filepath . $filename, "a");
        if (!$this->file_pointer) {
            exit("Error: Something went wrong with the logfile" . PHP_EOL);
        }
    }

    /**
     * Returns the full path to the logfile
     * @return String the path to the logfile
     */
    public function getLogPath() {
        return $this->log_path;
    }

    /**
     * Generates colored outputs
     * @param String $message
     * @return String
     */
    public static function colorOutput($message) {
        $pattern[0] = "^0";
        $replacement[0] = "\33[0m";
        $pattern[1] = "^1";
        $replacement[1] = "\33[0;31m";
        $pattern[2] = "^2";
        $replacement[2] = "\33[1;32m";
        $pattern[3] = "^3";
        $replacement[3] = "\33[0;33m";
        $pattern[4] = "^4";
        $replacement[4] = "\33[0;34m";
        $pattern[5] = "^5";
        $replacement[5] = "\33[0;36m";
        $pattern[6] = "^6";
        $replacement[6] = "\33[1;35m";
        $pattern[7] = "^7";
        $replacement[7] = "\33[1;37m";
        $pattern[8] = "^8";
        $replacement[8] = "\33[0;37m";
        $pattern[9] = "^9";
        $replacement[9] = "\33[1;30m";

        $message = str_replace($pattern, $replacement, $message);
        $end = "\33[0m";
        return $message . $end;
    }
}
