<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 **/

namespace MAM\Plugins\Statistics;

use MAM\Daemon\Core\Utils;

/**
 * Statistics Class
 * Handle the Internal MAM Statistics System !stats,!aliases,!session,!serverstats
 *
 * @author Manu
 * @license Creative Commons BY-NC-SA 3.0 (http://www.creativecommons.org/licenses/by-nc-sa/3.0/)
 * @version 1
 *
 */
class Statistics extends Utils {

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV('statistics', 'statsstring', '^2Kills: <KILLS> ^1Deaths: <DEATHS> ^3K/D: <KPD> ^2time: <ONLINETIME> min ^3K/M: <KPM> ^2HS: <HEADSHOTS> ^7TKs: <TEAMKILLS>');
        $this->mod->setDefaultCV('statistics', 'whisperstats', 1);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerCommand('session', false, 'commandSession', $this);
        $this->mod->registerCommand('stats', false, 'commandStats', $this);
        $this->mod->registerCommand('serverstats', false, 'commandServerstats', $this);
        $this->mod->registerCommand('resetplayerstats', false, 'commandResetplayerstats', $this);
        $this->mod->registerCommand('resetmystats', false, 'commandResetmystats', $this);

        $this->mod->registerEvent('playerJoined', 'eventPlayerJoined', $this);
        $this->mod->registerEvent('playerQuit', 'eventPlayerQuit', $this);

        $this->initUsers();
    }

    /**
     * This Event triggers if a new player joins
     * The new guid is passed to the addalias function
     *
     * @param String $guid 32 hex chars long guid of the player
     */
    public function eventPlayerJoined($guid) {
        $this->initUser($guid);
    }

    /**
     * This Event triggers if player leave the server
     *
     * @param String $guid 32 hex chars long guid
     */
    public function eventPlayerQuit($guid) {
        $this->saveStats($guid);
    }

    /**
     * This Command write the Stats to the chat
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandStats($guid, $parameters) {
        $this->writeStatsStringToChat($guid, $this->getGlobalStats($guid));
    }

    /**
     * This Command write the temporal Stats of the actual seesion to the chat
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandSession($guid, $parameters) {
        $this->writeStatsStringToChat($guid, $this->getSessionStats($guid));
    }

    /**
     * This Command write the serverstats to the chat
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandServerstats($guid, $parameters) {
        $this->writeStatsStringToChat($guid, $this->getOverallStats());
    }

    /**
     * This command resets the stats of a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  boolean     false if player not found
     */
    public function commandResetplayerstats($guid, $parameters) {
        $parameters = implode(' ', $parameters);

        $player = $this->mod->findPlayerGuid($parameters);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString('playerNotFound', array('<SEARCH>'), array($parameters)));
            return false;
        }

        $this->resetStats($player);
        $this->players[$guid]->say($this->mod->getLngString('PlayerStatsResetted', '<PLAYER_NAME>', $this->players[$player]->getName()));
    }

    /**
     * This command resets the stats of the executing player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandResetmystats($guid, $parameters) {
        $this->resetStats($guid);
        $this->players[$guid]->say($this->mod->getLngString('YourStatsResetted'));
    }

    /**
     * get session Statistics
     *
     * @param String $guid
     * @return mixed
     */
    protected function getSessionStats($guid) {
        return $this->getStatsOfPlayer($guid);
    }

    /**
     * get overall session Statistics
     *
     * @return array
     */
    protected function getOverallSessionStats() {
        $overall = self::getEmptyStat();
        foreach (array_keys($this->players) as $guid) {
            $overall = self::addStats($overall, $this->getStatsOfPlayer($guid));
        }
        $overall['lastseen'] = 0;
        return $overall;
    }

    /**
     * Write Statistics to the Chat
     *
     * @param string $guid
     * @param array $stats
     */
    protected function writeStatsStringToChat($guid, $stats) {
        $search = array(
            '<PLAYER_NAME>',
            '<KILLS>',
            '<DEATHS>',
            '<KPD>',
            '<KPM>',
            '<HPK>',
            '<TEAMKILLS>',
            '<TEAMDEATHS>',
            '<SELFKILLS>',
            '<ONLINETIME>',
            '<HEADSHOTS>',
        );

        $name = $this->players[$guid]->getName();
        $kills = $stats['kills'];
        $deaths = $stats['deaths'];
        $time = \MAM\Daemon\Libraries\Helper\Helper::convertSeconds($stats['time'], '%a <DAYS>, %h <HOURS>');
        $headshots = $stats['headshots'];
        $teamkills = $stats['teamkills'];
        $teamdeaths = $stats['teamdeaths'];
        $selfkills = $stats['selfkills'];

        $replace = array($name,
            $kills,
            $deaths,
            ($deaths == 0) ? '--' : round($kills / $deaths, 2),
            round($kills / $stats['time'], 2),
            ($kills == 0) ? '--' : round($headshots / $kills, 2),
            $teamkills,
            $teamdeaths,
            $selfkills,
            $time,
            $headshots,
        );

        $say = str_replace($search, $replace, $this->mod->getCV('statistics', 'statsstring'));

        if ($this->mod->getCV('statistics', 'whisperstats')) {
            $this->players[$guid]->say($say);
        } else {
            $this->rcon->rconSay($say);
        }
    }

    /**
     * adds the Statistics
     *
     * @param array $stats
     * @return array
     */
    protected static function addStats() {
        $return = self::getEmptyStat();
        $keys = array_keys(self::getEmptyStat());
        $lastseen = array();
        foreach (func_get_args() as $stat) {
            foreach ($keys as $key) {
                $return[$key] += $stat[$key];
                //  $lastseen[] = $stat['lastseen'];
            }
        }
        //  $return['lastseen'] = max($lastseen);
        return $return;
    }

    /**
     * get empty Statistics
     *
     * @return array
     */
    protected static function getEmptyStat() {
        return array('kills' => 0, 'deaths' => 0, 'headshots' => 0, 'teamkills' => 0, 'teamdeaths' => 0, 'selfkills' => 0, 'time' => 0, 'lastseen' => 0);
        //return array('kills' => 0, 'deaths' => 0, 'headshots' => 0, 'teamkills' => 0, 'teamdeaths' => 0, 'selfkills' => 0);
    }

    /**
     * Destruct the class
     */
    public function __destruct() {
        array_map(array($this, 'saveStats'), array_keys($this->players));
    }

    /**
     * get statistics of a player
     *
     * @param String $guid
     * @return array
     */
    protected function getStatsOfPlayer($guid) {
        $stats = self::getEmptyStat();
        $pl = &$this->players[$guid];

        $stats['kills'] = $pl->kills;
        $stats['deaths'] = $pl->deaths;
        $stats['headshots'] = $pl->headshots;
        $stats['teamkills'] = $pl->teamkills;
        $stats['teamdeaths'] = $pl->teamdeaths;
        $stats['selfkills'] = $pl->selfkills;
        $stats['time'] = time() - $pl->joined;
        $stats['lastseen'] = time();



        return $stats;
    }

    /**
     * save statistics of a player
     *
     * @param String $guid
     */
    protected function saveStats($guid) {
        $current = $this->getStatsOfPlayer($guid);

        $stats = $this->database->row('stats', array('user' => $this->players[$guid]->getUserId()));

        $stats = array(
            'kills' => $stats->kills + $current['kills'],
            'deaths' => $stats->deaths + $current['deaths'],
            'headshots' => $stats->headshots + $current['headshots'],
            'teamkills' => $stats->teamkills + $current['teamkills'],
            'teamdeaths' => $stats->teamdeaths + $current['teamdeaths'],
            'selfkills' => $stats->selfkills + $current['selfkills'],
        );

        $this->database->update('stats', $stats, array('user', '=', $this->players[$guid]->getUserId()));
    }

    /**
     * Adds an alias to the database
     * @param String $guid
     * @param String $name
     */
    protected function initUser($guid) {
        $user = $this->database->row('stats', array('user' => $this->players[$guid]->getUserId()));

        if (empty($user)) {
            $this->database->insert('stats', array('user' => $this->players[$guid]->getUserId()));
        }
    }

    /**
     * Exec the 'addAlias' function for every guid on the server
     */
    public function initUsers() {
        foreach (array_keys($this->players) as $guid) {
            $this->initUser($guid);
        }
    }

    /**
     * get global statistics of all players
     *
     * @param String $guid
     * @return mixed
     */
    protected function getGlobalStats($guid) {
        $this->database->setFetchType(\PDO::FETCH_ASSOC);

        $stats = $this->database->row('stats', array('user' => $this->players[$guid]->getUserId()));
        $var = $this->players[$guid]->getPlayerInfos();
        $stats['time'] = (int) $var->onlinetime;
        $stats['lastseen'] = 0;

        return self::addStats($this->getStatsOfPlayer($guid), $stats);
    }

    /**
     * get overall statistics
     *
     * @return mixed
     */
    protected function getOverallStats() {
        $sql = 'SELECT sum(stats.kills)      AS kills,
                       sum(stats.deaths)     AS deaths,
                       sum(stats.headshots)  AS headshots,
                       sum(stats.teamkills)  AS teamkills,
                       sum(stats.teamdeaths) AS teamdeaths,
                       sum(stats.selfkills)  AS selfkills,
                       sum(user.onlinetime)      AS time,
                       0               AS lastseen
                FROM <TABLE> AS stats, <PFX>user as user';

        $this->database->setFetchType(\PDO::FETCH_ASSOC);
        $stats = $this->database->row('stats', $sql);


        $sql = "SELECT sum(user.onlinetime)      AS time,
                        0                        AS lastseen
                FROM <TABLE> as user";

        $this->database->setFetchType(\PDO::FETCH_ASSOC);
        $user = $this->database->row('user', $sql);

        $stats_full = array_merge($stats, $user);

        return self::addStats($stats_full, $this->getOverallSessionStats());
    }

    /**
     * reset the statistics of a player
     *
     * @param string $guid
     */
    protected function resetStats($guid) {
        $stats = array(
            'kills' => 0,
            'deaths' => 0,
            'headshots' => 0,
            'teamkills' => 0,
            'teamdeaths' => 0,
            'selfkills' => 0,
        );

        $this->database->update('stats', $stats, array('user', '=', $this->players[$guid]->getUserId()));
    }
}
